@@ -61,6 +61,11 @@ struct _GnomeDEntryEdit {
   GtkWidget *terminal_button;  
 
   GtkWidget *icon_entry;
+  
+  GtkWidget *translations;
+  GtkWidget *transl_lang_entry;
+  GtkWidget *transl_name_entry;
+  GtkWidget *transl_comment_entry;
 };
 
 struct _GnomeDEntryEditClass {
@@ -103,12 +108,24 @@ void        gnome_dentry_edit_set_dentry (GnomeDEntryEdit * dee,
 					  GnomeDesktopEntry * dentry);
 
 /* Generate a dentry based on the contents of the display */
+GnomeDesktopEntry * gnome_dentry_edit_get_dentry(GnomeDEntryEdit * dee);
+/*XXX: whoops!!!!, this call is left here just for binary
+  compatibility, it should be gnome_dentry_edit_get_dentry*/
 GnomeDesktopEntry * gnome_dentry_get_dentry(GnomeDEntryEdit * dee);
 
 /* Return an allocated string, you need to g_free it. */
 gchar *     gnome_dentry_edit_get_icon   (GnomeDEntryEdit * dee);
 gchar *     gnome_dentry_edit_get_name   (GnomeDEntryEdit * dee);
 
+/* These are accessor functions for the widgets that make up the
+   GnomeDEntryEdit widget. */
+GtkWidget * gnome_dentry_get_name_entry      (GnomeDEntryEdit * dee);
+GtkWidget * gnome_dentry_get_comment_entry   (GnomeDEntryEdit * dee);
+GtkWidget * gnome_dentry_get_exec_entry      (GnomeDEntryEdit * dee);
+GtkWidget * gnome_dentry_get_tryexec_entry   (GnomeDEntryEdit * dee);
+GtkWidget * gnome_dentry_get_doc_entry       (GnomeDEntryEdit * dee);
+GtkWidget * gnome_dentry_get_icon_entry      (GnomeDEntryEdit * dee);
+
 END_GNOME_DECLS
    
 #endif /* GNOME_DENTRY_EDIT_H */
