@@ -318,6 +318,28 @@ path_max (void)
 #endif
 }
 
+/*
+  Use ttydefault definitions, if they exist, otherwise
+  make something up ...
+*/
+#ifndef TTYDEF_IFLAG
+	/*ICRNL|IXON;*/
+#define TTYDEF_IFLAG (BRKINT | ICRNL | IMAXBEL | IXON | IXANY)
+#endif
+#ifndef TTYDEF_OFLAG
+	/*OPOST|ONLCR|NL0|CR0|TAB0|BS0|VT0|FF0;*/
+#define TTYDEF_OFLAG (OPOST | ONLCR)
+#endif
+#ifndef TTYDEF_LFLAG
+	/*ISIG|ICANON|IEXTEN|ECHO|ECHOE|ECHOK|ECHOCTL|ECHOKE;*/
+#define TTYDEF_LFLAG (ECHO | ICANON | ISIG | IEXTEN | ECHOE | ECHOKE | ECHOCTL)
+#endif
+#ifndef TTYDEF_CFLAG
+	/*EXTB|CS8|CREAD|HUPCL;*/
+	/* B38400 isnt portable EXTB isn't really either, B9600 though  .. */
+#define TTYDEF_CFLAG (B9600 | CREAD | CS8 | HUPCL)
+#endif
+
 static int
 open_ptys (int utmp, int wtmp)
 {
@@ -325,10 +347,10 @@ open_ptys (int utmp, int wtmp)
 	int status, master_pty, slave_pty;
 	pty_info *p;
 	int result;
-	int size;
 	pid_t savedUid;
 	gid_t savedGid;
 	struct group *group_info;
+	struct termios term;
 	
 	term_name = ((char *)alloca (path_max())) + 1;
 
@@ -346,7 +368,65 @@ open_ptys (int utmp, int wtmp)
 	setgid (pwent->pw_gid);
 	setuid (pwent->pw_uid);
 #endif
-	status = openpty (&master_pty, &slave_pty, term_name, NULL, NULL);
+
+	/*
+	 *	openpty assumes POSIX termios so this should be portable.
+	 *	Don't change this to a structure init - POSIX doesn't say
+	 *	anything about field order.
+	 */
+	memset(&term, 0, sizeof(term));
+
+	term.c_iflag = TTYDEF_IFLAG;
+	term.c_oflag = TTYDEF_OFLAG;
+	term.c_cflag = TTYDEF_CFLAG;
+	term.c_lflag = TTYDEF_LFLAG;
+#ifdef N_TTY
+	/* should really be a check for c_line, but maybe this is good enough */
+	term.c_line = N_TTY;
+#endif
+
+	/* These two may overlap so set them first */
+	term.c_cc[VTIME] =  0;
+	term.c_cc[VMIN] = 0;
+	
+	/* Now set the characters. This is of course a religious matter
+	   but we use the defaults, with erase bound to the key gnome-terminal
+	   maps */
+	   
+	term.c_cc[VINTR] = 'C'-64;
+	term.c_cc[VQUIT] = '\\'-64;
+	term.c_cc[VERASE] = 'H'-64;
+	term.c_cc[VKILL] =  'U'-64;
+	term.c_cc[VEOF] =  'D'-64;
+#ifdef VSWTC
+	term.c_cc[VSWTC] = 0;
+#endif
+	term.c_cc[VSTART] = 'Q'-64;
+	term.c_cc[VSTOP] = 'S'-64;
+	term.c_cc[VSUSP] =  'Z'-64;
+	term.c_cc[VEOL] = 0;
+	
+	/*
+	 *	Extended stuff.
+	 */
+	 
+#ifdef VREPRINT	
+	term.c_cc[VREPRINT] = 'R'-64;
+#endif
+#ifdef VDISCARD	
+	term.c_cc[VDISCARD] = 'U'-64;
+#endif
+#ifdef VWERASE
+	term.c_cc[VWERASE] = 'W'-64;
+#endif	
+#ifdef VLNEXT
+	term.c_cc[VLNEXT] =  'V'-64;
+#endif
+#ifdef VEOL2	
+	term.c_cc[VEOL2] = 0;
+#endif
+
+	status = openpty (&master_pty, &slave_pty, term_name, &term, NULL);
 	setuid(savedUid);
 	setgid(savedGid);
 
@@ -498,7 +578,7 @@ main (int argc, char *argv [])
 	if (pwent)
 		login_name = pwent->pw_name;
 	else {
-		sprintf (login_name_buffer, "#%d", getuid ());
+		sprintf (login_name_buffer, "#%u", (unsigned int)getuid ());
 		login_name = login_name_buffer;
 	}
 
